!MNH_LIC Copyright 2021-2021 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
!     ###################
      MODULE MODI_NHOA_COUPL_n
!     ###################
!
INTERFACE 
!
      SUBROUTINE NHOA_COUPL_n(KDAD,PTSTEP,KMI,KTCOUNT,KKU)
!
INTEGER,          INTENT(IN)    :: KDAD     !  Number of the DAD model
REAL,             INTENT(IN)    :: PTSTEP   !  Time step
INTEGER,          INTENT(IN)    :: KMI      ! model number
INTEGER,          INTENT(IN)    :: KKU
INTEGER,          INTENT(IN)    :: KTCOUNT  !  Temporal loop COUNTer
                                            ! (=1 at the segment beginning)
!
END SUBROUTINE NHOA_COUPL_n
END INTERFACE
END MODULE MODI_NHOA_COUPL_n
!
!     ####################################################################
SUBROUTINE NHOA_COUPL_n(KDAD,PTSTEP,KMI,KTCOUNT,KKU)
!     ####################################################################
!!
!!    PURPOSE
!!    -------
!   To compute the flux at the O-A interface in the auto-coupling O-A LES case
!
!!**  METHOD
!!    ------
!!      
!!    EXTERNAL
!!    --------
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!!      Module MODD_PARAMETERS: JPHEXT,JPVEXT
!!      Module MODD_FIELD$n : XUT,XVT,XWT,XRT,XTHT,XPABST
!!      Module MODD_REF$n   : XRHODJ, XRVREF,XTHVREF, XRHODREF
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!   JL Redelsperger 03/2021   Version 0 
!!    MODIFICATIONS
!!    -------------
!  P. Wautelet 17/12/2021: bugfix: set IIU and IJU values
!!-----------------------------------------------------------------------------
!
!*      0.   DECLARATIONS
!            ------------
USE MODE_ll
USE MODE_MODELN_HANDLER
!
USE MODD_PARAMETERS
USE MODD_NESTING
USE MODD_CST
USE MODD_REF, ONLY: XRHODREFZ
USE MODD_REF_n        ! modules relative to the outer model $n
USE MODD_FIELD_n
USE MODD_METRICS_n
USE MODD_CONF
USE MODD_PARAM_n
USE MODD_TURB_n
USE MODD_DYN_n, ONLY : LOCEAN
USE MODD_OCEANH 
!
IMPLICIT NONE
!
!*       0.1   declarations of arguments
!
INTEGER,          INTENT(IN)    :: KDAD     !  Number of the DAD model
REAL,             INTENT(IN)    :: PTSTEP   !  Time step
INTEGER,          INTENT(IN)    :: KMI      ! model number
INTEGER,          INTENT(IN)    :: KKU      ! 
!
!
!*       0.2   declarations of local variables
!
INTEGER                :: IIU,IJU
INTEGER :: IKE
INTEGER                :: ILBX,ILBY,ILBX2,ILBY2
INTEGER,          INTENT(IN)    :: KTCOUNT  !  Temporal loop COUNTer
                                            ! (=1 at the segment beginning)
!
INTEGER           :: JRR,JSV          !  Loop index
!
INTEGER :: IINFO_ll, IDIMX, IDIMY
! surface variables: wind, current, Temp
REAL, DIMENSION(:,:), ALLOCATABLE :: ZCOUPUA,ZCOUPVA,ZCOUPTA
REAL, DIMENSION(:,:), ALLOCATABLE :: ZCOUPUO,ZCOUPVO,ZCOUPTO
!surf flux  local work space
REAL, DIMENSION(:,:), ALLOCATABLE :: ZCOUPTFL,ZCOUPUFL,ZCOUPVFL,ZCOUPQFL
REAL                              :: ZCDN ! CONSTANT DRAG COEFF
CHARACTER(LEN=4)                    :: ZINIT_TYPE
!
!---Coupled OA MesoNH----------------------------------------------------------------------------
!*      0.   INITIALISATION
!            --------------
IIU = SIZE(XRHODJ,1)
IJU = SIZE(XRHODJ,2)
ZCDN = 1.2E-3
! allocate sfc variable local array
ALLOCATE(ZCOUPUA(IIU,IJU))
ALLOCATE(ZCOUPVA(IIU,IJU))
ALLOCATE(ZCOUPTA(IIU,IJU))
ALLOCATE(ZCOUPUO(IIU,IJU))
ALLOCATE(ZCOUPVO(IIU,IJU))
ALLOCATE(ZCOUPTO(IIU,IJU))
ALLOCATE(ZCOUPTFL(IIU,IJU))
ALLOCATE(ZCOUPUFL(IIU,IJU))
ALLOCATE(ZCOUPVFL(IIU,IJU))
ALLOCATE(ZCOUPQFL(IIU,IJU))
! loading values in ocean sfc
IKE=KKU-JPVEXT
ZCOUPUO(:,:)= XUT(:,:,IKE)
ZCOUPVO(:,:)= XVT(:,:,IKE)
ZCOUPTO(:,:)= XTHT(:,:,IKE)
!
! we are going to the atmos model i.e. Model 1
CALL GOTO_MODEL(KDAD)
! compute gradient between ocean & atmosphere
ZCOUPUA(:,:)= XUT(:,:,2)-ZCOUPUO(:,:)
ZCOUPVA(:,:)= XVT(:,:,2)-ZCOUPVO(:,:)
ZCOUPTA(:,:)= XTHT(:,:,2)-ZCOUPTO(:,:)
!!!!!!!!!!!!
!  Simple neutral bulk flux with Z0 fixed
ZCOUPTFL(:,:) = -ZCDN*SQRT(ZCOUPUA(:,:)**2 +ZCOUPVA(:,:)**2) * ZCOUPTA(:,:)&
                *XCPD*XRHODREFZ(2)        ! in W/M2 (as WASP)              
ZCOUPUFL(:,:) = -ZCDN*XRHODREFZ(2)* SQRT(ZCOUPUA(:,:)**2 +ZCOUPVA(:,:)**2) * ZCOUPUA(:,:)
ZCOUPVFL(:,:) = -ZCDN*XRHODREFZ(2)* SQRT(ZCOUPUA(:,:)**2 +ZCOUPVA(:,:)**2) * ZCOUPVA(:,:)
!  EVAP-PR TO BE ADDED HERE 
ZCOUPQFL(:,:)= 0.
! keep in W/M2, N/m2, ..
XSSUFL= ZCOUPUFL
XSSVFL= ZCOUPVFL
XSSTFL= ZCOUPTFL
XSSRFL= ZCOUPQFL
!
!  back in the ocean model  
!oce flux are directly computed in phys_param routine
! same sign & unit fluxes at top of ocean model & bottom of atmospheric model
!  rho_atmos * (w'u')_atmos = rho_ocean * (u'w')_ocean
!  rho_atmos *Cp_atmos* (T'w')_atmos = rho_ocean * CP_ocean * (T'w')_ocean
!
CALL GOTO_MODEL(KMI)
DEALLOCATE(ZCOUPUA,ZCOUPVA,ZCOUPUO,ZCOUPVO,ZCOUPTA,ZCOUPTO)
DEALLOCATE(ZCOUPTFL)
!
END SUBROUTINE NHOA_COUPL_n

